<?php

namespace SGalinski\Lfeditor\Utility;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use Exception;
use RuntimeException;
use TYPO3\CMS\Core\Configuration\ConfigurationManager;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * includes special typo3 methods
 */
class Typo3Lib {
	const PATH_LOCAL_EXT = 'typo3conf/ext/';

	const PATH_GLOBAL_EXT = 'typo3/ext/';

	const PATH_SYS_EXT = 'typo3/sysext/';

	const PATH_L10N = 'typo3conf/l10n/';

	/**
	 * checks the file location type
	 *
	 * @param string $file
	 * @return string
	 */
	public static function checkFileLocation($file) {
		if (strpos($file, self::PATH_LOCAL_EXT) !== FALSE) {
			return 'local';
		}

		if (strpos($file, self::PATH_GLOBAL_EXT) !== FALSE) {
			return 'global';
		}

		if (strpos($file, self::PATH_SYS_EXT) !== FALSE) {
			return 'system';
		}

		if (strpos($file, self::PATH_L10N) !== FALSE) {
			return 'l10n';
		}

		return '';
	}

	/**
	 * converts an absolute or relative typo3 style (EXT:) file path
	 *
	 * @throws Exception raised, if the conversion fails
	 * @param string $file absolute file or an typo3 relative file (EXT:)
	 * @param boolean $mode generate to relative(false) or absolute file
	 * @return string converted file path
	 */
	public static function transTypo3File($file, $mode) {
		$extType['local'] = self::PATH_LOCAL_EXT;
		$extType['global'] = self::PATH_GLOBAL_EXT;
		$extType['system'] = self::PATH_SYS_EXT;

		// relative to absolute
		if ($mode) {
			if (strpos($file, 'EXT:') === FALSE) {
				throw new RuntimeException('no typo3 relative path "' . $file . '"');
			}

			$cleanFile = SgLib::trimPath('EXT:', $file);
			/** @var  string[] $extType */
			foreach ($extType as $type) {
				$path = self::fixFilePath(PATH_site . '/' . $type . '/' . $cleanFile);
				if (is_dir(\dirname($path))) {
					return $path;
				}
			}

			throw new RuntimeException('cant convert typo3 relative file "' . $file . '"');
		}

		// absolute to relative
		/** @var  string[] $extType */
		foreach ($extType as $type) {
			if (strpos($file, $type) === FALSE) {
				continue;
			}

			return 'EXT:' . SgLib::trimPath($type, SgLib::trimPath(PATH_site, $file));
		}

		throw new RuntimeException('cant convert absolute file "' . $file . '"');
	}

	/**
	 * generates portable file paths
	 *
	 * @param string $file file
	 * @return string fixed file
	 */
	public static function fixFilePath($file) {
		return GeneralUtility::fixWindowsFilePath(str_replace('//', '/', $file));
	}

	/**
	 * Adds configuration line to AdditionalConfiguration.php.
	 *
	 * @param string $configLine line to be added.
	 * @return void
	 * @throws \InvalidArgumentException
	 */
	public static function writeLineToAdditionalConfiguration($configLine) {
		/** @var ConfigurationManager $configurationManager */
		$configurationManager = GeneralUtility::makeInstance(ConfigurationManager::class);
		$additionalConfigurationFilePath = $configurationManager->getAdditionalConfigurationFileLocation();
		SgLib::appendToPHPFile($additionalConfigurationFilePath, $configLine);
	}
}
